from django.utils.translation import gettext_lazy as _

from gas.sites import site


site.register_role('staff', _('Users with access to gas control panel.'))
site.register_role('admins', _('Users with access to everithing inside control panel.'))

site.register_urls('', 'gas.gas.core.urls')

site.register_urls('users', 'gas.gas.users.urls')
site.register_menu(
    name='users',
    label=_("Users"),
    icon="",
    url="gas:user_list",
    roles=('admins',),
)



import json

from django import forms
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse
from django.test import TestCase
from django.test.client import RequestFactory
from django.views.generic import View, FormView, TemplateView
from django.contrib import messages
from django.contrib.messages.storage import default_storage

from model_bakery import baker

from gas import views as gviews
from gas import gas_settings
from gas.forms import UserForm
from gas.gas.users.views import UserList


class AjaxCommandTestCase(TestCase):
    def test_ajaxcommands_view(self):
        class SampleAjaxView(gviews.AjaxCommandsMixin, View):
            def do_test(self):
                return self.render_json({'success': True})

        class SampleForm(forms.Form):
            field = forms.CharField()

        class SampleAjaxWithPostView(gviews.AjaxCommandsMixin, FormView):
            form_class = SampleForm

            def render_to_response(self, context, **response_kwargs):
                return HttpResponse("OK")

            def form_valid(self, form):
                return self.form_invalid(form)

        view = SampleAjaxView.as_view()
        post_view = SampleAjaxWithPostView.as_view()
        request_factory = RequestFactory()

        # Command with corresponding "do_" method
        request = request_factory.post('some_url', {
            'command': 'test',
        })
        response = view(request)
        self.assertEqual(response['Content-Type'], 'application/json')
        data = json.loads(response.content)
        self.assertTrue(data['success'])

        # Command with no corresponding "do_" method returns BadRequest
        request = request_factory.post('some_url', {
            'command': 'missing_test',
        })
        response = view(request)
        self.assertEqual(response.status_code, 400)  # Bad Request

        # With no command process normal POST request
        request = request_factory.post('some_url', {
            'not_command': 'test',
        })
        response = view(request)
        self.assertEqual(response.status_code, 405)  # Not Allowed
        request = request_factory.post('some_url', {
            'field': 'test',
        })
        response = post_view(request)
        self.assertEqual(response.status_code, 200)


class GASMixinTestCase(TestCase):
    def setUp(self):
        self.admin_user = baker.make(
            'auth.User',
            is_superuser=True,
        )
        self.admin_role_user = baker.make(
            'auth.User',
            user_roles__role='admins'
        )
        self.staff_role_user = baker.make(
            'auth.User',
            user_roles__role='staff'
        )
        self.no_role_user = baker.make(
            'auth.User',
            is_superuser=False,
        )
        self.request_factory = RequestFactory()

    def test_dispatch(self):
        class SampleView(gviews.GASMixin, View):
            """
                GAS default authentication, allowed superusers and users
                with admins role.
            """
            def get(self, request, *args, **kwargs):
                return HttpResponse("OK")

        view = SampleView.as_view()
        request = self.request_factory.get('some_url')

        # superusers allowed
        request.user = self.admin_user
        response = view(request)
        self.assertEqual(response.status_code, 200)

        # admins role allowed
        request.user = self.admin_role_user
        response = view(request)
        self.assertEqual(response.status_code, 200)

        # others not allowed
        request.user = self.no_role_user
        response = view(request)
        self.assertEqual(response.status_code, 302)
        request.user = self.staff_role_user
        response = view(request)
        self.assertEqual(response.status_code, 302)

        class SampleStaffView(gviews.GASMixin, View):
            """
                Allowed superusers and users with admins and staff roles.
            """
            roles = ('staff',)

            def get(self, request, *args, **kwargs):
                return HttpResponse("OK")

        view = SampleStaffView.as_view()
        request = self.request_factory.get('some_url')

        # superusers allowed
        request.user = self.admin_user
        response = view(request)
        self.assertEqual(response.status_code, 200)

        # admins role allowed
        request.user = self.admin_role_user
        response = view(request)
        self.assertEqual(response.status_code, 200)

        # staff role allowed
        request.user = self.staff_role_user
        response = view(request)
        self.assertEqual(response.status_code, 200)

        # others not allowed
        request.user = self.no_role_user
        response = view(request)
        self.assertEqual(response.status_code, 302)

    def test_form_valid(self):
        class SampleForm(forms.Form):
            field = forms.CharField()

        class SampleFormView(gviews.GASMixin, FormView):
            form_class = SampleForm
            success_message = 'success message'
            continue_url = 'continue'
            success_url = 'success'

        view = SampleFormView.as_view()

        # Normal workflow
        request = self.request_factory.post('some_url', {
            'field': 'test',
        })
        request.user = self.admin_user
        request._messages = default_storage(request)
        response = view(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, 'success')
        msgs = [
            (m.level, m.message)
            for m in messages.get_messages(request)
        ]
        self.assertIn((messages.SUCCESS, 'success message'), msgs)

        # Save and continue
        request = self.request_factory.post('some_url', {
            'field': 'test',
            'save_and_continue': '1',
        })
        request.user = self.admin_user
        request._messages = default_storage(request)
        response = view(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, 'continue')

    def test_context_data(self):
        class SampleView(gviews.GASMixin, TemplateView):
            title = 'title'
            header_title = 'header title'
            help_text = 'help text'
            breadcrumbs = [
                ('url', 'label'),
            ]
            template_name = 'gas/base.html'

        view = SampleView.as_view()
        request = self.request_factory.get('some_url')
        request.user = self.admin_user
        response = view(request)
        self.assertEqual(response.context_data['title'], SampleView.title)
        self.assertEqual(response.context_data['header_title'], SampleView.header_title)
        self.assertEqual(response.context_data['help_text'], SampleView.help_text)
        self.assertEqual(response.context_data['base_template'], SampleView.base_template)
        self.assertEqual(response.context_data['breadcrumbs'], SampleView.breadcrumbs)
        self.assertEqual(response.context_data['gas_title'], gas_settings.TITLE)
        self.assertEqual(response.context_data['logo_static_url'], gas_settings.LOGO)
        self.assertEqual(response.context_data['css'], gas_settings.MEDIA['css'])
        self.assertEqual(response.context_data['js'], gas_settings.MEDIA['js'])

    def test_no_header_title(self):
        class NoHeaderTitleSampleView(gviews.GASMixin, TemplateView):
            title = 'title'
            template_name = 'gas/base.html'

        view = NoHeaderTitleSampleView.as_view()
        request = self.request_factory.get('some_url')
        request.user = self.admin_user
        response = view(request)
        self.assertEqual(response.context_data['title'], NoHeaderTitleSampleView.title)
        self.assertEqual(response.context_data['header_title'], NoHeaderTitleSampleView.title)

        # test error with save_and_continue and no continue_url
        class SampleForm(forms.Form):
            field = forms.CharField()

        class NoContinueUrlSampleView(gviews.GASMixin, FormView):
            form_class = SampleForm
            success_url = 'success'

        view = NoContinueUrlSampleView.as_view()
        request = self.request_factory.post('some_url', {
            'field': 'test',
            'save_and_continue': '1',
        })
        request.user = self.admin_user

        with self.assertRaises(ImproperlyConfigured):
            response = view(request)

    def test_extra_css(self):
        class SampleView(gviews.GASMixin, TemplateView):
            template_name = 'test.html'

        view = SampleView.as_view()
        request = self.request_factory.get('some_url')

        request.user = self.admin_user
        response = view(request)
        self.assertEqual(response.status_code, 200)

        gas_settings.EXTRA_MEDIA = None
        response = view(request)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('test.css', response.context_data['css'])
        self.assertNotIn('test.js', response.context_data['js'])

        gas_settings.EXTRA_MEDIA = {'css': ['test.css']}
        response = view(request)
        self.assertEqual(response.status_code, 200)
        self.assertIn('test.css', response.context_data['css'])
        self.assertNotIn('test.js', response.context_data['js'])

        gas_settings.EXTRA_MEDIA = {'js': ['test.js']}
        response = view(request)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('test.css', response.context_data['css'])
        self.assertIn('test.js', response.context_data['js'])

        gas_settings.EXTRA_MEDIA = {'css': ['test.css'], 'js': ['test.js']}
        response = view(request)
        self.assertEqual(response.status_code, 200)
        self.assertIn('test.css', response.context_data['css'])
        self.assertIn('test.js', response.context_data['js'])


class GASListTestCase(TestCase):
    def test_filtered(self):
        request_factory = RequestFactory()

        baker.make(
            'auth.User', username='testuser',
            user_roles__role='staff'
        )
        admin_user = baker.make(
            'auth.User',
            username='anothertestuser',
            user_roles__role='admins',
            is_superuser=True,
        )
        baker.make('auth.User', username='anotherone')

        view = UserList.as_view()

        # default listing with all users
        request = request_factory.get('some_url')
        request.user = admin_user
        response = view(request)
        self.assertEqual(response.context_data['user_list'].count(), 3)

        # filter by username
        request = request_factory.get('some_url', {
            'username': 'test',
        })
        request.user = admin_user
        response = view(request)
        self.assertEqual(response.context_data['user_list'].count(), 2)

        # filter by roles
        request = request_factory.get('some_url', {
            'roles': ['staff'],
        })
        request.user = admin_user
        response = view(request)
        self.assertEqual(response.context_data['user_list'].count(), 1)

        # filter_form_class not required
        class NotFiltered(gviews.GASListView):
            model = User

        view = NotFiltered.as_view()
        request = request_factory.get('some_url')
        request.user = admin_user
        response = view(request)
        self.assertEqual(response.context_data['user_list'].count(), 3)


class GASCreateTestCase(TestCase):
    def test_popup(self):
        request_factory = RequestFactory()

        admin_user = baker.make(
            'auth.User',
            username='admin',
            is_superuser=True,
        )

        class SampleCreateView(gviews.GASCreateView):
            model = User
            form_class = UserForm
            success_url = 'success'

        view = SampleCreateView.as_view()

        # returning from popup
        request = request_factory.post('some_url', {
            '_popup': '1',

            'username': 'testuser',
            'first_name': 'test',
            'last_name': 'user',
            'email': 'email@example.org',
            'is_active': '1',
        })
        request.user = admin_user
        request._messages = default_storage(request)
        response = view(request)
        self.assertIn(b'dismissAddAnotherPopup', response.content)

        # normal workflow, redirect to success_url
        request = request_factory.post('some_url', {
            'username': 'testuser2',
            'first_name': 'test2',
            'last_name': 'user2',
            'email': 'email2@example.org',
            'is_active': '1',
        })
        request.user = admin_user
        request._messages = default_storage(request)
        response = view(request)
        self.assertEqual(response.status_code, 302)

        # is_popup on context_data
        request = request_factory.get('some_url', {
            '_popup': '1',
        })
        request.user = admin_user
        response = view(request)
        self.assertEqual(response.context_data['is_popup'], True)


class GASDeleteTestCase(TestCase):
    def test_context_data(self):
        request_factory = RequestFactory()
        user = baker.make(User, username='testuser')
        admin_user = baker.make(
            'auth.User',
            username='admin',
            is_superuser=True,
        )

        class SampleDeleteView(gviews.GASDeleteView):
            model = User
            success_url = 'success'
            confirmation_text = 'delete {object.username}'
            deleted_text = '{object.username} deleted'
        
        view = SampleDeleteView.as_view()

        request = request_factory.get('some_url')
        request.user = admin_user
        response = view(request, pk=user.pk)
        ctx = response.context_data

        self.assertEqual(response.status_code, 200)
        self.assertEqual(ctx['confirmation_text'], 'delete testuser')
        self.assertEqual(response.context_data['cancel_url'], 'success')
        self.assertEqual(len(ctx['deleted_objects']), 1)
        self.assertEqual(ctx['deleted_model_count']['users'], 1)

        request = request_factory.post('some_url', pk=user.pk)
        request.user = admin_user
        request._messages = default_storage(request)
        response = view(request, pk=user.pk)

        self.assertEqual(response.status_code, 302)
        msgs = [
            (m.level, m.message)
            for m in messages.get_messages(request)
        ]
        self.assertIn((messages.SUCCESS, 'testuser deleted'), msgs)



from django.contrib.auth.forms import AdminPasswordChangeForm
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404
from django.views.generic import FormView
from django.urls import reverse, reverse_lazy

from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

import gas.views as gviews
from gas.forms import UserForm

from .forms import UserFilterForm


class BaseMixin:
    model = User
    form_class = UserForm
    filter_form_class = UserFilterForm
    roles = ('admins',)

    def get_success_url(self):
        return reverse('gas:user_list')

    def get_continue_url(self):
        return reverse('gas:user_update', kwargs={'pk': self.object.pk})


class UserList(BaseMixin, gviews.GASListView):
    template_name = 'gas/users/user_list.html'
    title = _('Users')
    help_text = _('Manage users and set roles.')
    actions = [
        (reverse_lazy('gas:user_create'), 'fa-plus', _('New user')),
    ]

    def get_breadcrumbs(self):
        return (
            (None, _('Users')),
        )


class CreateUser(BaseMixin, gviews.GASCreateView):
    title = _('Create user')
    success_message = _('User created')

    def get_breadcrumbs(self):
        return (
            (reverse('gas:user_list'), _('Users')),
            (None, _('Create')),
        )


class UpdateUser(BaseMixin, gviews.GASUpdateView):
    title = _('Update user')
    success_message = _('User updated')

    def get_breadcrumbs(self):
        return (
            (reverse('gas:user_list'), _('Users')),
            (reverse('gas:user_update', args=(self.object.pk,)), self.object.username),
            (None, _('Update')),
        )


class ChangePasswordUser(BaseMixin, gviews.GASMixin, FormView):
    template_name = 'gas/base_form.html'
    form_class = AdminPasswordChangeForm
    success_message = _('User password updated')

    @cached_property
    def user(self):
        return get_object_or_404(User, pk=self.kwargs['pk'])

    def get_title(self):
        return _('Change {username} user password').format(username=self.user.username)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['user'] = self.user
        return kwargs

    def form_valid(self, form):
        form.save()
        return super().form_valid(form)

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx.update({
            'target_user': self.user,
        })
        return ctx

    def get_breadcrumbs(self):
        return (
            (reverse('gas:user_list'), _('Users')),
            (reverse('gas:user_update', args=(self.user.pk,)), self.user.username),
            (None, _('Change password')),
        )


class DeleteUser(BaseMixin, gviews.GASDeleteView):
    title = _('Delete user')
    success_message = _('User deleted')

    def get_breadcrumbs(self):
        return (
            (reverse('gas:user_list'), _('Users')),
            (reverse('gas:user_update', args=(self.object.pk,)), self.object.username),
            (None, _('Delete')),
        )



from django.contrib.auth.models import User
from django.test import TestCase

from model_bakery import baker

from gas import forms
from gas.gas.users import forms as user_forms


class UserFormsTestCase(TestCase):
    def test_user_form(self):
        user = baker.make('auth.User', first_name='name')
        data = {
            'first_name': 'changed name',
            'last_name': user.last_name,
            'username': user.username,
            'email': user.email,
            'is_active': user.is_active,
            'roles': ['staff',],
        }
        form = forms.UserForm(instance=user, data=data)
        self.assertTrue(form.is_valid())
        new_user = form.save()
        self.assertEqual(new_user.email, user.email)
        self.assertEqual(new_user.first_name, 'changed name')
        self.assertTrue(
            new_user.user_roles.filter(role='staff').exists()
        )
        self.assertFalse(
            new_user.user_roles.filter(role='admins').exists()
        )


class UserFilterFormTestCase(TestCase):
    def test_filter_form(self):
        user = baker.make(User,
            username='testname',
            user_roles__role='staff',
            is_active=True,
        )
        users = User.objects.all()
        
        # filter by username with match
        data = {
            'username': 'test',
        }
        form = user_forms.UserFilterForm(data)
        self.assertTrue(form.is_valid())
        user_in_queryset = form.filter(users).filter(pk=user.id).exists()
        self.assertTrue(user_in_queryset)

        # filter by username without match
        data = {
            'username': 'nonexistant',
        }
        form = user_forms.UserFilterForm(data)
        self.assertTrue(form.is_valid())
        user_in_queryset = form.filter(users).filter(pk=user.id).exists()
        self.assertFalse(user_in_queryset)

        # filter by is_active with match
        data = {
            'is_active': 'true',
        }
        form = user_forms.UserFilterForm(data)
        self.assertTrue(form.is_valid())
        user_in_queryset = form.filter(users).filter(pk=user.id).exists()
        self.assertTrue(user_in_queryset)

        # filter by is_active without match
        data = {
            'is_active': 'false',
        }
        form = user_forms.UserFilterForm(data)
        self.assertTrue(form.is_valid())
        filtered_qs = form.filter(users)
        user_in_queryset = form.filter(users).filter(pk=user.id).exists()
        self.assertFalse(user_in_queryset)

        # filter by role without match
        data = {
            'roles': ['admins'],
        }
        form = user_forms.UserFilterForm(data)
        self.assertTrue(form.is_valid())
        user_in_queryset = form.filter(users).filter(pk=user.id).exists()
        self.assertFalse(user_in_queryset)



# Generated by Django 3.1.4 on 2020-12-15 17:37

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', models.CharField(choices=[], max_length=150, verbose_name='role')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_roles', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'user role',
                'verbose_name_plural': 'user roles',
            },
        ),
    ]



from django.contrib.auth.views import logout_then_login
from django.urls import path

from . import views


urlpatterns = [
    path('login/', views.GASLoginView.as_view(), name='login'),
    path('logout/', logout_then_login, {'login_url': 'gas:login'}, name='logout'),
    path('change-password/', views.GASPasswordChangeView.as_view(), name='change_password'),
    path('', views.Index.as_view(), name='index'),
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _
from django.db.models.signals import post_migrate


class GASConfig(AppConfig):
    name = 'gas'
    verbose_name = _('GAS')

    def ready(self):
        from .sites import site
        super().ready()
        site.autodiscover()



from django.contrib.auth.views import LoginView, PasswordChangeView
from django.shortcuts import resolve_url
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic import TemplateView

from gas.views import GASMixin


class GASLoginView(LoginView):
    template_name = "gas/login.html"

    def get_success_url(self):
        url = self.get_redirect_url()
        return url or resolve_url('gas:index')


class GASPasswordChangeView(GASMixin, PasswordChangeView):
    template_name = 'gas/base_form.html'
    success_url = reverse_lazy('gas:index')
    title = _('Change your password')
    success_message = _('Password changed.')

class Index(GASMixin, TemplateView):
    main_menu = 'index'
    template_name = "gas/index.html"
    roles = ('staff',)



from django import forms
from django.utils.translation import gettext_lazy as _

from gas.forms import BaseFilterForm
from gas.sites import site


class UserFilterForm(BaseFilterForm):
    username = forms.CharField(label=_('Username'), required=False)
    is_active = forms.NullBooleanField(label=_('Active'), required=False)
    roles = forms.MultipleChoiceField(
        label=_('Roles'), choices=site.role_choices, required=False,
    )

    def filter(self, qs):
        qs = super().filter(qs)

        username = self.cleaned_data['username']
        if username:
            qs = qs.filter(username__icontains=username)

        is_active = self.cleaned_data['is_active']
        if is_active is not None:
            qs = qs.filter(is_active=is_active)

        roles = self.cleaned_data['roles']
        if roles:
            qs = qs.filter(user_roles__role__in=roles)

        return qs



from django.db import models
from django.utils.translation import gettext_lazy as _

from gas.sites import site


class UserRole(models.Model):
    role = models.CharField(_('role'), max_length=150, choices=site.role_choices)
    user = models.ForeignKey(
        'auth.User', verbose_name=_('user'), related_name='user_roles',
        on_delete=models.CASCADE)

    class Meta:
        verbose_name = _('user role')
        verbose_name_plural = _('user roles')

    def __str__(self):
        return self.role

    @property
    def description(self):
        return site.get_role_description(self.role)



from django.conf import settings


TITLE = getattr(settings, 'GAS_TITLE', 'GAS')
LOGO = getattr(settings, 'GAS_LOGO', 'gas/css/img/logo.svg')

MEDIA = getattr(settings, 'GAS_MEDIA', {
    'css': [
        'vendor/font-awesome/css/all.css',
        'vendor/select2/css/select2.css',
        'gas/css/gas.css',
    ],
    'js': [
        'vendor/jquery/dist/jquery.min.js',
        'vendor/jquery.formset.js',
        'vendor/select2/js/select2.full.min.js',
        'gas/js/add_popups.js',
        'gas/js/gas.js',
    ],
})

EXTRA_MEDIA = getattr(settings, 'GAS_EXTRA_MEDIA', None)
